@@ -158,43 +158,62 @@ macro_rules! loop_rand {
     }};
 }
 
-// NB: On AMD processor families < 0x17, we want to unconditionally disable RDRAND
-// and RDSEED. Executing these instructions on these processors can return
-// non-random data (0) while also reporting a success.
-//
-// See:
-// * https://github.com/systemd/systemd/issues/11810
-// * https://lore.kernel.org/all/776cb5c2d33e7fd0d2893904724c0e52b394f24a.1565817448.git.thomas.lendacky@amd.com/
-//
-// We take extra care to do so even if `-Ctarget-features=+rdrand` have been
-// specified, in order to prevent users from shooting themselves in their feet.
+#[inline(always)]
+fn authentic_amd() -> bool {
+    let cpuid0 = unsafe { arch::__cpuid(0) };
+    matches!(
+        (cpuid0.ebx, cpuid0.ecx, cpuid0.edx),
+        (0x68747541, 0x444D4163, 0x69746E65)
+    )
+}
+
+#[inline(always)]
+fn amd_family(cpuid1: &arch::CpuidResult) -> u32 {
+    ((cpuid1.eax >> 8) & 0xF) + ((cpuid1.eax >> 20) & 0xFF)
+}
+
+#[inline(always)]
+fn has_rdrand(cpuid1: &arch::CpuidResult) -> bool {
+    const FLAG: u32 = 1 << 30;
+    cpuid1.ecx & FLAG == FLAG
+}
+
+#[inline(always)]
+fn has_rdseed() -> bool {
+    const FLAG: u32 = 1 << 18;
+    unsafe { arch::__cpuid(7).ebx & FLAG == FLAG }
+}
+
+/// NB: On AMD processor families < 0x17, we want to unconditionally disable RDRAND
+/// and RDSEED. Executing these instructions on these processors can return
+/// non-random data (0) while also reporting a success.
+///
+/// See:
+/// * https://github.com/systemd/systemd/issues/11810
+/// * https://lore.kernel.org/all/776cb5c2d33e7fd0d2893904724c0e52b394f24a.1565817448.git.thomas.lendacky@amd.com/
+///
+/// We take extra care to do so even if `-Ctarget-features=+rdrand` have been
+/// specified, in order to prevent users from shooting themselves in their feet.
+const FIRST_GOOD_AMD_FAMILY: u32 = 0x17;
+
 macro_rules! is_available {
     ("rdrand") => {{
-        const FLAG: u32 = 1 << 30;
-        let cpuid0 = unsafe { arch::__cpuid(0) };
-        if let (0x68747541, 0x444D4163, 0x69746E65) = (cpuid0.ebx, cpuid0.ecx, cpuid0.edx) {
-            // This is an AuthenticAMD chip.
+        if authentic_amd() {
             let cpuid1 = unsafe { arch::__cpuid(1) };
-            let family = ((cpuid1.eax >> 8) & 0xF) + ((cpuid1.eax >> 20) & 0xFF);
-            (cpuid1.ecx & FLAG == FLAG) && (family >= 0x17)
+            has_rdrand(&cpuid1) && amd_family(&cpuid1) >= FIRST_GOOD_AMD_FAMILY
         } else {
-            cfg!(target_feature="rdrand") || unsafe { arch::__cpuid(1).ecx & FLAG == FLAG }
+            cfg!(target_feature = "rdrand") || has_rdrand(&unsafe { arch::__cpuid(1) })
         }
     }};
     ("rdseed") => {{
-        const FLAG : u32 = 1 << 18;
-        let cpuid0 = unsafe { arch::__cpuid(0) };
-        if let (0x68747541, 0x444D4163, 0x69746E65) = (cpuid0.ebx, cpuid0.ecx, cpuid0.edx) {
-            let cpuid1 = unsafe { arch::__cpuid(1) };
-            let family = ((cpuid1.eax >> 8) & 0xF) + ((cpuid1.eax >> 20) & 0xFF);
-            (family >= 0x17) && unsafe { arch::__cpuid(7).ebx & FLAG == FLAG }
+        if authentic_amd() {
+            amd_family(&unsafe { arch::__cpuid(1) }) >= FIRST_GOOD_AMD_FAMILY && has_rdseed()
         } else {
-            cfg!(target_feature="rdrand") || unsafe { arch::__cpuid(7).ebx & FLAG == FLAG }
+            cfg!(target_feature = "rdrand") || has_rdseed()
         }
     }};
 }
 
-
 macro_rules! impl_rand {
     ($gen:ident, $feat:tt, $step16: path, $step32:path, $step64:path,
      maxstep = $maxstep:path, maxty = $maxty: ty) => {
@@ -205,8 +224,8 @@ macro_rules! impl_rand {
             /// instruction necessary for this generator to operate. If the instruction is not
             /// supported, an error is returned.
             pub fn new() -> Result<Self, ErrorCode> {
-                if cfg!(target_env="sgx") {
-                    if cfg!(target_feature=$feat) {
+                if cfg!(target_env = "sgx") {
+                    if cfg!(target_feature = $feat) {
                         Ok($gen(()))
                     } else {
                         Err(ErrorCode::UnsupportedInstruction)
@@ -306,7 +325,13 @@ macro_rules! impl_rand {
                     ) -> Result<(), ErrorCode> {
                         let mut word;
                         let mut buffer: &[u8] = &[];
-                        while !left.is_empty() {
+                        loop {
+                            if left.is_empty() {
+                                if right.is_empty() {
+                                    break;
+                                }
+                                ::core::mem::swap(&mut left, &mut right);
+                            }
                             if buffer.is_empty() {
                                 word =
                                     unsafe { loop_rand!($feat, $maxty, $maxstep) }?.to_ne_bytes();
@@ -318,9 +343,6 @@ macro_rules! impl_rand {
                             buffer = leftover;
                             left = dest_leftover;
                             copy_dest.copy_from_slice(copy_src);
-                            if left.is_empty() {
-                                ::core::mem::swap(&mut left, &mut right);
-                            }
                         }
                         Ok(())
                     }
@@ -479,39 +501,48 @@ mod test {
         });
     }
 
+    #[repr(C, align(8))]
+    struct FillBuffer([u8; 64]);
+
     #[test]
     fn fill_fills_all_bytes() {
         let _ = RdRand::new().map(|mut r| {
-            let mut peach;
-            let mut banana;
-            let mut start = 0;
-            let mut end = 128;
-            'outer: while start < end {
-                banana = [0; 128];
+            let mut test_buffer;
+            let mut fill_buffer = FillBuffer([0; 64]); // make sure buffer is aligned to 8-bytes...
+            let test_cases = [
+                (0, 64), // well aligned
+                (8, 64), // well aligned
+                (0, 64), // well aligned
+                (5, 64), // left is non-empty, right is empty.
+                (0, 63), // left is empty, right is non-empty.
+                (5, 63), // left and right both are non-empty.
+                (5, 61), // left and right both are non-empty.
+                (0, 8),   // 1 word-worth of data, aligned.
+                (1, 9),   // 1 word-worth of data, misaligned.
+                (0, 7),   // less than 1 word of data.
+                (1, 7),   // less than 1 word of data.
+            ];
+            'outer: for &(start, end) in &test_cases {
+                test_buffer = [0; 64];
                 for _ in 0..512 {
-                    peach = [0; 128];
-                    r.fill_bytes(&mut peach[start..end]);
-                    for (b, p) in banana.iter_mut().zip(peach.iter()) {
+                    fill_buffer.0 = [0; 64];
+                    r.fill_bytes(&mut fill_buffer.0[start..end]);
+                    for (b, p) in test_buffer.iter_mut().zip(fill_buffer.0.iter()) {
                         *b = *b | *p;
                     }
-                    if (&banana[start..end]).iter().all(|x| *x != 0) {
+                    if (&test_buffer[start..end]).iter().all(|x| *x != 0) {
                         assert!(
-                            banana[..start].iter().all(|x| *x == 0),
+                            test_buffer[..start].iter().all(|x| *x == 0),
                             "all other values must be 0"
                         );
                         assert!(
-                            banana[end..].iter().all(|x| *x == 0),
+                            test_buffer[end..].iter().all(|x| *x == 0),
                             "all other values must be 0"
                         );
-                        if start < 17 {
-                            start += 1;
-                        } else {
-                            end -= 3;
-                        }
                         continue 'outer;
                     }
                 }
-                panic!("wow, we broke it? {} {} {:?}", start, end, &banana[..])
+                panic!("wow, we broke it? {} {} {:?}", start, end, &test_buffer[..])
             }
         });
     }
